#!/bin/bash

# Percentage of Memory reserved for TCP Sockets
M=75

PMEM=0
[ -z ${MAX_CONCURRENT} ] && MAX_CONCURRENT=100
[ -z ${MAXTHREADS} ] && MAXTHREADS=0
[ -z ${MAX_FDS} ] && MAX_FDS=0

TMP_SYSCTL="/tmp/sysctl_.conf"


_IFS="${IFS}"
IFS=$'\t'
s=0
SPACE="$[ s += 30 ],$[ s += 15 ],$[ s += 20 ],$[ s += 25 ],$[ s += 25 ]"
declare -A _SYSCTLV
declare -A TCP_MEM_BYTES
declare -A TCP_RMEM
declare -A TCP_WMEM
declare -A E_PORT
_F=$[ 100 / M ]

__err_report() {
    echo "Error on line $1"
}

# trap 'err_report $LINENO' ERR
#trap 'err_report ${BASH_SOURCE}:${FUNCNAME}:${LINENO}' ERR

trap 'echo "error: ${FUNCNAME}:${LINENO}"' ERR


_SYSCTLV[net.core.netdev_max_backlog]=""
_SYSCTLV[net.core.rmem_default]=""
_SYSCTLV[net.core.rmem_max]=""
_SYSCTLV[net.core.somaxconn]=""
_SYSCTLV[net.core.wmem_default]=""
_SYSCTLV[net.core.wmem_max]=""
_SYSCTLV[net.ipv4.ip_local_port_range]=""
_SYSCTLV[net.ipv4.tcp_abort_on_overflow]=""
_SYSCTLV[net.ipv4.tcp_adv_win_scale]=""
_SYSCTLV[net.ipv4.tcp_fin_timeout]=""
_SYSCTLV[net.ipv4.tcp_keepalive_intvl]=""
_SYSCTLV[net.ipv4.tcp_keepalive_probes]=""
_SYSCTLV[net.ipv4.tcp_keepalive_time]=""
_SYSCTLV[net.ipv4.tcp_max_syn_backlog]=""
_SYSCTLV[net.ipv4.tcp_mem]=""
_SYSCTLV[net.ipv4.tcp_mtu_probing]=""
_SYSCTLV[net.ipv4.tcp_rfc1337]=""
_SYSCTLV[net.ipv4.tcp_rmem]=""
_SYSCTLV[net.ipv4.tcp_tw_recycle]=""
_SYSCTLV[net.ipv4.tcp_tw_reuse]=""
_SYSCTLV[net.ipv4.tcp_window_scaling]=""
_SYSCTLV[net.ipv4.tcp_wmem]=""
_SYSCTLV[net.ipv4.tcp_max_syn_backlog]=""
_SYSCTLV[vm.swappiness]=""
_SYSCTLV[vm.overcommit_memory]=""
_SYSCTLV[vm.overcommit_ratio]=""

_SYSCTLV[net.ipv4.netfilter.ip_conntrack_generic_timeout]=""
_SYSCTLV[net.ipv4.netfilter.ip_conntrack_tcp_timeout_established]=""
_SYSCTLV[net.netfilter.nf_conntrack_max]=""


for K in "${!_SYSCTLV[@]}"; do _SYSCTLV[$K]=`sysctl -n $K`; done
#for K in "${!_SYSCTLV[@]}"; do echo $K=${_SYSCTLV[$K]}; done | sort -n

PROJECT_LOAD()
{
	MAXTHREADS=$[ 2 * MAX_CONCURRENT ]
	MAX_FDS=$[ 2 * MAXTHREADS ]
	echo "Projected Load:"
	echo MAXTHREADS: ${MAXTHREADS}
	echo MAX_FDS: ${MAX_FDS}
}

PHYSICAL_MEMORY()
{
# get physical memory available in the system
	PAGESIZE=`getconf PAGESIZE`
	M_PAGES=`getconf _PHYS_PAGES`
	PMEM=$((`getconf _PHYS_PAGES` * PAGESIZE ))
	echo "SYSTEM_MEMORY (PAGES): ${PMEM} ( ${M_PAGES} )"
}

OPTIMIZE_SOCKET_BUFFERS()
{

	# Limit maximum memory utilization for TCP to 25% of the system memory
	# on a 2GB system this will be 500M
	TCP_MEM[2]=$[ M_PAGES / _F ]
	
	# Set the pressure level to 66% of maximum allocated TCP memory
	# on a 2GB system this will be 330M
	TCP_MEM[1]=$[ TCP_MEM[2] / 3 * 2 ]

	# Set the low water-mark to 50% of TCP memory at pressure level
	# on a 2GB system this will be 150M
	TCP_MEM[0]=$[ TCP_MEM[1] / 2 ]
	
	# concatenate the values for TCP_MEM
	_SYSCTLV[net.ipv4.tcp_mem]="${TCP_MEM[0]} ${TCP_MEM[1]} ${TCP_MEM[2]}"
	
	# set the defaults for rmem and wmem

	SKT_DEF_PAGE=$[ TCP_MEM[1] / MAX_FDS ]

	_SYSCTLV[net.core.rmem_default]=$[ SKT_DEF_PAGE * PAGESIZE ]
	_SYSCTLV[net.core.wmem_default]=$[ SKT_DEF_PAGE * PAGESIZE ]
	
	# enable receive buffer auto-tuning, 
	# attempting to automatically size the buffer to match the size required by the path for full throughput.
	_SYSCTLV[net.ipv4.tcp_moderate_rcvbuf]=1

	# set maximum pages of memory per socket receive buffer
	SKT_MAX_PAGE=$[ TCP_MEM[2] / MAX_FDS ]
	
	_SYSCTLV[net.core.rmem_max]=$[ SKT_MAX_PAGE * PAGESIZE ] 
	_SYSCTLV[net.core.wmem_max]=$[ SKT_MAX_PAGE * PAGESIZE ]
	
	_SYSCTLV[net.ipv4.tcp_rmem]="${PAGESIZE} ${_SYSCTLV[net.core.rmem_default]} ${_SYSCTLV[net.core.rmem_max]}"
	_SYSCTLV[net.ipv4.tcp_wmem]="${PAGESIZE} ${_SYSCTLV[net.core.wmem_default]} ${_SYSCTLV[net.core.wmem_max]}"

	_SYSCTLV[net.core.somaxconn]=$[ MAX_CONCURRENT * 10 ]
	_SYSCTLV[net.core.netdev_max_backlog]=$[ MAX_CONCURRENT * 200 ]
	_SYSCTLV[net.ipv4.tcp_max_syn_backlog]=$[ MAX_CONCURRENT * 10 ]
	_SYSCTLV[net.ipv4.tcp_mtu_probing]=1
	_SYSCTLV[net.ipv4.ip_local_port_range]="10000 65000"
	_SYSCTLV[net.ipv4.tcp_fin_timeout]=15
	_SYSCTLV[net.ipv4.tcp_rfc1337]=1
	_SYSCTLV[net.ipv4.tcp_abort_on_overflow]=1
	_SYSCTLV[net.ipv4.tcp_adv_win_scale]=1
	_SYSCTLV[net.ipv4.tcp_keepalive_intvl]=75
	_SYSCTLV[net.ipv4.tcp_keepalive_probes]=9
	_SYSCTLV[net.ipv4.tcp_keepalive_time]=900
	_SYSCTLV[net.ipv4.tcp_tw_recycle]=1
	_SYSCTLV[net.ipv4.tcp_tw_reuse]=1
	_SYSCTLV[net.ipv4.tcp_window_scaling]=1
	_SYSCTLV[vm.swappiness]=1
	_SYSCTLV[vm.overcommit_memory]=2
	_SYSCTLV[vm.overcommit_ratio]=60
	_SYSCTLV[net.ipv4.tcp_rfc1337]=0
	_SYSCTLV[net.ipv4.tcp_slow_start_after_idle]=0
	_SYSCTLV[net.ipv4.tcp_sack]=1
	_SYSCTLV[net.ipv4.tcp_ecn]=1
 
	_SYSCTLV[net.ipv4.netfilter.ip_conntrack_generic_timeout]=120
	_SYSCTLV[net.ipv4.netfilter.ip_conntrack_tcp_timeout_established]=54000
	_SYSCTLV[net.netfilter.nf_conntrack_max]=131072

 
 
 
	RECEIVE_SOCKET_BUFFERS="${_SYSCTLV[net.core.rmem_default]}"
	SEND_SOCKET_BUFFERS="${_SYSCTLV[net.core.wmem_default]}"
	
	[ ${RECEIVE_SOCKET_BUFFERS} -gt 262144 ] && RECEIVE_SOCKET_BUFFERS=262144
	[ ${SEND_SOCKET_BUFFERS} -gt 262144 ] && SEND_SOCKET_BUFFERS=262144
	
#	for K in "${!_SYSCTLV[@]}"; do sysctl -w $K=${_SYSCTLV[$K]}; done 

return;
}



BUILD_SYSCTL()
{

> ${TMP_SYSCTL}

cat <<- _EOF | tee -a ${TMP_SYSCTL}

###############
#        	 MAX_CONCURRENT=${MAX_CONCURRENT}
#       	        MAX_FDS=${MAX_FDS}
#   	TOTAL SYSTEM MEMORY=${PMEM}
# MAX TCP MEMORY ALLOCATION=$[ TCP_MEM[2] * PAGESIZE ]
#           DEFAULT RCV_BUF=${_SYSCTLV[net.core.rmem_default]}
#           DEFAULT SND_BUF=${_SYSCTLV[net.core.wmem_default]}
#               MAX RCV_BUF=${_SYSCTLV[net.core.rmem_max]}
#               MAX SND_BUF=${_SYSCTLV[net.core.wmem_max]}
###############
_EOF


	for K in "${!_SYSCTLV[@]}"
	do 
		echo $K=${_SYSCTLV[$K]}; 
#		sysctl -w $K=${_SYSCTLV[$K]}
	done | sort | tee -a ${TMP_SYSCTL}

	echo "#######xx######" >> ${TMP_SYSCTL}
}

LOAD_SYSCTL()
{

	[ "x${TMP_SYSCTL}" == "x" ] && return;
	[ -f ${TMP_SYSCTL} ] || return;
	
	sysctl --load=${TMP_SYSCTL}
}


MANGO()
{

cat <<- _EOF 
RECEIVE_SOCKET_BUFFERS="${RECEIVE_SOCKET_BUFFERS}"
SEND_SOCKET_BUFFERS="${RECEIVE_SOCKET_BUFFERS}"
_EOF

}

SET_UP_CONNTRACK()
{

	NF_CONNTRACK_MAX=${_SYSCTLV[net.netfilter.nf_conntrack_max]}
	HASHSIZE=$[ NF_CONNTRACK_MAX / 4]
	echo ${HASHSIZE} > /sys/module/nf_conntrack/parameters/hashsize

}


MAIN()
{
	PHYSICAL_MEMORY
	PROJECT_LOAD	
	OPTIMIZE_SOCKET_BUFFERS
	BUILD_SYSCTL
	LOAD_SYSCTL
	SET_UP_CONNTRACK
	
	[ "x${SYSTEM_INI}" != "x" ] && MANGO > ${SYSTEM_INI}
	
	echo ""
	
}

MAIN
